"""Windows-specific platform features"""
import ctypes
import platform
from OpenGL.platform import ctypesloader, baseplatform
import sys

if sys.hexversion < 0x2070000:
    vc = 'vc7'
elif sys.hexversion >= 0x3050000:
    vc = 'vc14'
elif sys.hexversion >= 0x3030000:
    vc = 'vc10'
else:
    vc = 'vc9'

def _size():
    return platform.architecture()[0].strip( 'bits' )
size = _size()

class Win32Platform( baseplatform.BasePlatform ):
    """Win32-specific platform implementation"""

    GLUT_GUARD_CALLBACKS = True
    @baseplatform.lazy_property
    def GL(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.windll, 'opengl32', mode = ctypes.RTLD_GLOBAL
            ) 
        except OSError as err:
            raise ImportError("Unable to load OpenGL library", *err.args)
    @baseplatform.lazy_property
    def GLU(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.windll, 'glu32', mode = ctypes.RTLD_GLOBAL
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def GLUT( self ):
        for possible in ('freeglut%s.%s'%(size,vc,), 'glut%s.%s'%(size,vc,)):
            # Prefer FreeGLUT if the user has installed it, fallback to the included 
            # GLUT if it is installed
            try:
                return ctypesloader.loadLibrary(
                    ctypes.windll, possible, mode = ctypes.RTLD_GLOBAL
                )
            except WindowsError:
                pass
        return None
    @baseplatform.lazy_property
    def GLE( self ):
        for libName in ('gle%s.%s'%(size,vc,), 'opengle%s.%s'%(size,vc,)):
            try:
                GLE = ctypesloader.loadLibrary( ctypes.cdll, libName )
                GLE.FunctionType = ctypes.CFUNCTYPE
                return GLE
            except WindowsError:
                pass
            else:
                break
        return None

    DEFAULT_FUNCTION_TYPE = staticmethod( ctypes.WINFUNCTYPE )
    # Win32 GLUT uses different types for callbacks and functions...
    GLUT_CALLBACK_TYPE = staticmethod( ctypes.CFUNCTYPE )
    GDI32 = ctypes.windll.gdi32
    @baseplatform.lazy_property
    def WGL( self ):
        return self.OpenGL
    @baseplatform.lazy_property
    def getExtensionProcedure( self ):
        wglGetProcAddress = self.OpenGL.wglGetProcAddress
        wglGetProcAddress.restype = ctypes.c_void_p
        return wglGetProcAddress

    GLUT_FONT_CONSTANTS = {
        'GLUT_STROKE_ROMAN': ctypes.c_void_p( 0),
        'GLUT_STROKE_MONO_ROMAN': ctypes.c_void_p( 1),
        'GLUT_BITMAP_9_BY_15': ctypes.c_void_p( 2),
        'GLUT_BITMAP_8_BY_13': ctypes.c_void_p( 3),
        'GLUT_BITMAP_TIMES_ROMAN_10': ctypes.c_void_p( 4),
        'GLUT_BITMAP_TIMES_ROMAN_24': ctypes.c_void_p( 5),
        'GLUT_BITMAP_HELVETICA_10': ctypes.c_void_p( 6),
        'GLUT_BITMAP_HELVETICA_12': ctypes.c_void_p( 7),
        'GLUT_BITMAP_HELVETICA_18': ctypes.c_void_p( 8),
    }


    def getGLUTFontPointer( self,constant ):
        """Platform specific function to retrieve a GLUT font pointer

        GLUTAPI void *glutBitmap9By15;
        #define GLUT_BITMAP_9_BY_15		(&glutBitmap9By15)

        Key here is that we want the addressof the pointer in the DLL,
        not the pointer in the DLL.  That is, our pointer is to the
        pointer defined in the DLL, we don't want the *value* stored in
        that pointer.
        """
        return self.GLUT_FONT_CONSTANTS[ constant ]

    @baseplatform.lazy_property
    def GetCurrentContext( self ):
        wglGetCurrentContext = self.GL.wglGetCurrentContext
        wglGetCurrentContext.restype = ctypes.c_void_p
        return wglGetCurrentContext

    def constructFunction(
        self,
        functionName, dll, 
        resultType=ctypes.c_int, argTypes=(),
        doc = None, argNames = (),
        extension = None,
        deprecated = False,
        module = None,
        force_extension = False,
        error_checker = None,
    ):
        """Override construct function to do win32-specific hacks to find entry points"""
        try:
            return super( Win32Platform, self ).constructFunction(
                functionName, dll,
                resultType, argTypes,
                doc, argNames,
                extension,
                deprecated,
                module,
                error_checker=error_checker,
            )
        except AttributeError:
            try:
                return super( Win32Platform, self ).constructFunction(
                    functionName, self.GDI32,
                    resultType, argTypes,
                    doc, argNames,
                    extension,
                    deprecated,
                    module,
                    error_checker=error_checker,
                )
            except AttributeError:
                return super( Win32Platform, self ).constructFunction(
                    functionName, dll,
                    resultType, argTypes,
                    doc, argNames,
                    extension,
                    deprecated,
                    module,
                    force_extension = True,
                    error_checker=error_checker,
                )
            



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLSC2 import _types as _cs
# End users want this...
from OpenGL.raw.GLSC2._types import *
from OpenGL.raw.GLSC2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLSC2_SC_VERSION_2_0'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLSC2,'GLSC2_SC_VERSION_2_0',error_checker=_errors._error_checker)
GL_ACTIVE_TEXTURE=_C('GL_ACTIVE_TEXTURE',0x84E0)
GL_ALIASED_LINE_WIDTH_RANGE=_C('GL_ALIASED_LINE_WIDTH_RANGE',0x846E)
GL_ALIASED_POINT_SIZE_RANGE=_C('GL_ALIASED_POINT_SIZE_RANGE',0x846D)
GL_ALPHA_BITS=_C('GL_ALPHA_BITS',0x0D55)
GL_ALWAYS=_C('GL_ALWAYS',0x0207)
GL_ARRAY_BUFFER=_C('GL_ARRAY_BUFFER',0x8892)
GL_ARRAY_BUFFER_BINDING=_C('GL_ARRAY_BUFFER_BINDING',0x8894)
GL_BACK=_C('GL_BACK',0x0405)
GL_BLEND=_C('GL_BLEND',0x0BE2)
GL_BLEND_COLOR=_C('GL_BLEND_COLOR',0x8005)
GL_BLEND_DST_ALPHA=_C('GL_BLEND_DST_ALPHA',0x80CA)
GL_BLEND_DST_RGB=_C('GL_BLEND_DST_RGB',0x80C8)
GL_BLEND_EQUATION=_C('GL_BLEND_EQUATION',0x8009)
GL_BLEND_EQUATION_ALPHA=_C('GL_BLEND_EQUATION_ALPHA',0x883D)
GL_BLEND_EQUATION_RGB=_C('GL_BLEND_EQUATION_RGB',0x8009)
GL_BLEND_SRC_ALPHA=_C('GL_BLEND_SRC_ALPHA',0x80CB)
GL_BLEND_SRC_RGB=_C('GL_BLEND_SRC_RGB',0x80C9)
GL_BLUE_BITS=_C('GL_BLUE_BITS',0x0D54)
GL_BUFFER_SIZE=_C('GL_BUFFER_SIZE',0x8764)
GL_BUFFER_USAGE=_C('GL_BUFFER_USAGE',0x8765)
GL_BYTE=_C('GL_BYTE',0x1400)
GL_CCW=_C('GL_CCW',0x0901)
GL_CLAMP_TO_EDGE=_C('GL_CLAMP_TO_EDGE',0x812F)
GL_COLOR_ATTACHMENT0=_C('GL_COLOR_ATTACHMENT0',0x8CE0)
GL_COLOR_BUFFER_BIT=_C('GL_COLOR_BUFFER_BIT',0x00004000)
GL_COLOR_CLEAR_VALUE=_C('GL_COLOR_CLEAR_VALUE',0x0C22)
GL_COLOR_WRITEMASK=_C('GL_COLOR_WRITEMASK',0x0C23)
GL_COMPRESSED_TEXTURE_FORMATS=_C('GL_COMPRESSED_TEXTURE_FORMATS',0x86A3)
GL_CONSTANT_ALPHA=_C('GL_CONSTANT_ALPHA',0x8003)
GL_CONSTANT_COLOR=_C('GL_CONSTANT_COLOR',0x8001)
GL_CONTEXT_LOST=_C('GL_CONTEXT_LOST',0x0507)
GL_CONTEXT_ROBUST_ACCESS=_C('GL_CONTEXT_ROBUST_ACCESS',0x90F3)
GL_CULL_FACE=_C('GL_CULL_FACE',0x0B44)
GL_CULL_FACE_MODE=_C('GL_CULL_FACE_MODE',0x0B45)
GL_CURRENT_PROGRAM=_C('GL_CURRENT_PROGRAM',0x8B8D)
GL_CURRENT_VERTEX_ATTRIB=_C('GL_CURRENT_VERTEX_ATTRIB',0x8626)
GL_CW=_C('GL_CW',0x0900)
GL_DECR=_C('GL_DECR',0x1E03)
GL_DECR_WRAP=_C('GL_DECR_WRAP',0x8508)
GL_DEPTH_ATTACHMENT=_C('GL_DEPTH_ATTACHMENT',0x8D00)
GL_DEPTH_BITS=_C('GL_DEPTH_BITS',0x0D56)
GL_DEPTH_BUFFER_BIT=_C('GL_DEPTH_BUFFER_BIT',0x00000100)
GL_DEPTH_CLEAR_VALUE=_C('GL_DEPTH_CLEAR_VALUE',0x0B73)
GL_DEPTH_COMPONENT16=_C('GL_DEPTH_COMPONENT16',0x81A5)
GL_DEPTH_FUNC=_C('GL_DEPTH_FUNC',0x0B74)
GL_DEPTH_RANGE=_C('GL_DEPTH_RANGE',0x0B70)
GL_DEPTH_TEST=_C('GL_DEPTH_TEST',0x0B71)
GL_DEPTH_WRITEMASK=_C('GL_DEPTH_WRITEMASK',0x0B72)
GL_DITHER=_C('GL_DITHER',0x0BD0)
GL_DONT_CARE=_C('GL_DONT_CARE',0x1100)
GL_DST_ALPHA=_C('GL_DST_ALPHA',0x0304)
GL_DST_COLOR=_C('GL_DST_COLOR',0x0306)
GL_DYNAMIC_DRAW=_C('GL_DYNAMIC_DRAW',0x88E8)
GL_ELEMENT_ARRAY_BUFFER=_C('GL_ELEMENT_ARRAY_BUFFER',0x8893)
GL_ELEMENT_ARRAY_BUFFER_BINDING=_C('GL_ELEMENT_ARRAY_BUFFER_BINDING',0x8895)
GL_EQUAL=_C('GL_EQUAL',0x0202)
GL_EXTENSIONS=_C('GL_EXTENSIONS',0x1F03)
GL_FALSE=_C('GL_FALSE',0)
GL_FASTEST=_C('GL_FASTEST',0x1101)
GL_FLOAT=_C('GL_FLOAT',0x1406)
GL_FRAMEBUFFER=_C('GL_FRAMEBUFFER',0x8D40)
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME=_C('GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME',0x8CD1)
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE=_C('GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE',0x8CD0)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL=_C('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL',0x8CD2)
GL_FRAMEBUFFER_BINDING=_C('GL_FRAMEBUFFER_BINDING',0x8CA6)
GL_FRAMEBUFFER_COMPLETE=_C('GL_FRAMEBUFFER_COMPLETE',0x8CD5)
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT=_C('GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT',0x8CD6)
GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS=_C('GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS',0x8CD9)
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT=_C('GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT',0x8CD7)
GL_FRAMEBUFFER_UNDEFINED=_C('GL_FRAMEBUFFER_UNDEFINED',0x8219)
GL_FRAMEBUFFER_UNSUPPORTED=_C('GL_FRAMEBUFFER_UNSUPPORTED',0x8CDD)
GL_FRONT=_C('GL_FRONT',0x0404)
GL_FRONT_AND_BACK=_C('GL_FRONT_AND_BACK',0x0408)
GL_FRONT_FACE=_C('GL_FRONT_FACE',0x0B46)
GL_FUNC_ADD=_C('GL_FUNC_ADD',0x8006)
GL_FUNC_REVERSE_SUBTRACT=_C('GL_FUNC_REVERSE_SUBTRACT',0x800B)
GL_FUNC_SUBTRACT=_C('GL_FUNC_SUBTRACT',0x800A)
GL_GENERATE_MIPMAP_HINT=_C('GL_GENERATE_MIPMAP_HINT',0x8192)
GL_GEQUAL=_C('GL_GEQUAL',0x0206)
GL_GREATER=_C('GL_GREATER',0x0204)
GL_GREEN_BITS=_C('GL_GREEN_BITS',0x0D53)
GL_GUILTY_CONTEXT_RESET=_C('GL_GUILTY_CONTEXT_RESET',0x8253)
GL_HIGH_FLOAT=_C('GL_HIGH_FLOAT',0x8DF2)
GL_HIGH_INT=_C('GL_HIGH_INT',0x8DF5)
GL_IMPLEMENTATION_COLOR_READ_FORMAT=_C('GL_IMPLEMENTATION_COLOR_READ_FORMAT',0x8B9B)
GL_IMPLEMENTATION_COLOR_READ_TYPE=_C('GL_IMPLEMENTATION_COLOR_READ_TYPE',0x8B9A)
GL_INCR=_C('GL_INCR',0x1E02)
GL_INCR_WRAP=_C('GL_INCR_WRAP',0x8507)
GL_INNOCENT_CONTEXT_RESET=_C('GL_INNOCENT_CONTEXT_RESET',0x8254)
GL_INT=_C('GL_INT',0x1404)
GL_INVALID_ENUM=_C('GL_INVALID_ENUM',0x0500)
GL_INVALID_FRAMEBUFFER_OPERATION=_C('GL_INVALID_FRAMEBUFFER_OPERATION',0x0506)
GL_INVALID_OPERATION=_C('GL_INVALID_OPERATION',0x0502)
GL_INVALID_VALUE=_C('GL_INVALID_VALUE',0x0501)
GL_INVERT=_C('GL_INVERT',0x150A)
GL_KEEP=_C('GL_KEEP',0x1E00)
GL_LEQUAL=_C('GL_LEQUAL',0x0203)
GL_LESS=_C('GL_LESS',0x0201)
GL_LINEAR=_C('GL_LINEAR',0x2601)
GL_LINEAR_MIPMAP_LINEAR=_C('GL_LINEAR_MIPMAP_LINEAR',0x2703)
GL_LINEAR_MIPMAP_NEAREST=_C('GL_LINEAR_MIPMAP_NEAREST',0x2701)
GL_LINES=_C('GL_LINES',0x0001)
GL_LINE_LOOP=_C('GL_LINE_LOOP',0x0002)
GL_LINE_STRIP=_C('GL_LINE_STRIP',0x0003)
GL_LINE_WIDTH=_C('GL_LINE_WIDTH',0x0B21)
GL_LINK_STATUS=_C('GL_LINK_STATUS',0x8B82)
GL_LOSE_CONTEXT_ON_RESET=_C('GL_LOSE_CONTEXT_ON_RESET',0x8252)
GL_LOW_FLOAT=_C('GL_LOW_FLOAT',0x8DF0)
GL_LOW_INT=_C('GL_LOW_INT',0x8DF3)
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS=_C('GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS',0x8B4D)
GL_MAX_FRAGMENT_UNIFORM_VECTORS=_C('GL_MAX_FRAGMENT_UNIFORM_VECTORS',0x8DFD)
GL_MAX_RENDERBUFFER_SIZE=_C('GL_MAX_RENDERBUFFER_SIZE',0x84E8)
GL_MAX_TEXTURE_IMAGE_UNITS=_C('GL_MAX_TEXTURE_IMAGE_UNITS',0x8872)
GL_MAX_TEXTURE_SIZE=_C('GL_MAX_TEXTURE_SIZE',0x0D33)
GL_MAX_VARYING_VECTORS=_C('GL_MAX_VARYING_VECTORS',0x8DFC)
GL_MAX_VERTEX_ATTRIBS=_C('GL_MAX_VERTEX_ATTRIBS',0x8869)
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS=_C('GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS',0x8B4C)
GL_MAX_VERTEX_UNIFORM_VECTORS=_C('GL_MAX_VERTEX_UNIFORM_VECTORS',0x8DFB)
GL_MAX_VIEWPORT_DIMS=_C('GL_MAX_VIEWPORT_DIMS',0x0D3A)
GL_MEDIUM_FLOAT=_C('GL_MEDIUM_FLOAT',0x8DF1)
GL_MEDIUM_INT=_C('GL_MEDIUM_INT',0x8DF4)
GL_MIRRORED_REPEAT=_C('GL_MIRRORED_REPEAT',0x8370)
GL_NEAREST=_C('GL_NEAREST',0x2600)
GL_NEAREST_MIPMAP_LINEAR=_C('GL_NEAREST_MIPMAP_LINEAR',0x2702)
GL_NEAREST_MIPMAP_NEAREST=_C('GL_NEAREST_MIPMAP_NEAREST',0x2700)
GL_NEVER=_C('GL_NEVER',0x0200)
GL_NICEST=_C('GL_NICEST',0x1102)
GL_NONE=_C('GL_NONE',0)
GL_NOTEQUAL=_C('GL_NOTEQUAL',0x0205)
GL_NO_ERROR=_C('GL_NO_ERROR',0)
GL_NO_ERROR=_C('GL_NO_ERROR',0)
GL_NUM_COMPRESSED_TEXTURE_FORMATS=_C('GL_NUM_COMPRESSED_TEXTURE_FORMATS',0x86A2)
GL_NUM_PROGRAM_BINARY_FORMATS=_C('GL_NUM_PROGRAM_BINARY_FORMATS',0x87FE)
GL_ONE=_C('GL_ONE',1)
GL_ONE_MINUS_CONSTANT_ALPHA=_C('GL_ONE_MINUS_CONSTANT_ALPHA',0x8004)
GL_ONE_MINUS_CONSTANT_COLOR=_C('GL_ONE_MINUS_CONSTANT_COLOR',0x8002)
GL_ONE_MINUS_DST_ALPHA=_C('GL_ONE_MINUS_DST_ALPHA',0x0305)
GL_ONE_MINUS_DST_COLOR=_C('GL_ONE_MINUS_DST_COLOR',0x0307)
GL_ONE_MINUS_SRC_ALPHA=_C('GL_ONE_MINUS_SRC_ALPHA',0x0303)
GL_ONE_MINUS_SRC_COLOR=_C('GL_ONE_MINUS_SRC_COLOR',0x0301)
GL_OUT_OF_MEMORY=_C('GL_OUT_OF_MEMORY',0x0505)
GL_PACK_ALIGNMENT=_C('GL_PACK_ALIGNMENT',0x0D05)
GL_POINTS=_C('GL_POINTS',0x0000)
GL_POLYGON_OFFSET_FACTOR=_C('GL_POLYGON_OFFSET_FACTOR',0x8038)
GL_POLYGON_OFFSET_FILL=_C('GL_POLYGON_OFFSET_FILL',0x8037)
GL_POLYGON_OFFSET_UNITS=_C('GL_POLYGON_OFFSET_UNITS',0x2A00)
GL_PROGRAM_BINARY_FORMATS=_C('GL_PROGRAM_BINARY_FORMATS',0x87FF)
GL_R8=_C('GL_R8',0x8229)
GL_RED=_C('GL_RED',0x1903)
GL_RED_BITS=_C('GL_RED_BITS',0x0D52)
GL_RENDERBUFFER=_C('GL_RENDERBUFFER',0x8D41)
GL_RENDERBUFFER_ALPHA_SIZE=_C('GL_RENDERBUFFER_ALPHA_SIZE',0x8D53)
GL_RENDERBUFFER_BINDING=_C('GL_RENDERBUFFER_BINDING',0x8CA7)
GL_RENDERBUFFER_BLUE_SIZE=_C('GL_RENDERBUFFER_BLUE_SIZE',0x8D52)
GL_RENDERBUFFER_DEPTH_SIZE=_C('GL_RENDERBUFFER_DEPTH_SIZE',0x8D54)
GL_RENDERBUFFER_GREEN_SIZE=_C('GL_RENDERBUFFER_GREEN_SIZE',0x8D51)
GL_RENDERBUFFER_HEIGHT=_C('GL_RENDERBUFFER_HEIGHT',0x8D43)
GL_RENDERBUFFER_INTERNAL_FORMAT=_C('GL_RENDERBUFFER_INTERNAL_FORMAT',0x8D44)
GL_RENDERBUFFER_RED_SIZE=_C('GL_RENDERBUFFER_RED_SIZE',0x8D50)
GL_RENDERBUFFER_STENCIL_SIZE=_C('GL_RENDERBUFFER_STENCIL_SIZE',0x8D55)
GL_RENDERBUFFER_WIDTH=_C('GL_RENDERBUFFER_WIDTH',0x8D42)
GL_RENDERER=_C('GL_RENDERER',0x1F01)
GL_REPEAT=_C('GL_REPEAT',0x2901)
GL_REPLACE=_C('GL_REPLACE',0x1E01)
GL_RESET_NOTIFICATION_STRATEGY=_C('GL_RESET_NOTIFICATION_STRATEGY',0x8256)
GL_RG=_C('GL_RG',0x8227)
GL_RG8=_C('GL_RG8',0x822B)
GL_RGB=_C('GL_RGB',0x1907)
GL_RGB565=_C('GL_RGB565',0x8D62)
GL_RGB5_A1=_C('GL_RGB5_A1',0x8057)
GL_RGB8=_C('GL_RGB8',0x8051)
GL_RGBA=_C('GL_RGBA',0x1908)
GL_RGBA4=_C('GL_RGBA4',0x8056)
GL_RGBA8=_C('GL_RGBA8',0x8058)
GL_SAMPLER_2D=_C('GL_SAMPLER_2D',0x8B5E)
GL_SAMPLES=_C('GL_SAMPLES',0x80A9)
GL_SAMPLE_ALPHA_TO_COVERAGE=_C('GL_SAMPLE_ALPHA_TO_COVERAGE',0x809E)
GL_SAMPLE_BUFFERS=_C('GL_SAMPLE_BUFFERS',0x80A8)
GL_SAMPLE_COVERAGE=_C('GL_SAMPLE_COVERAGE',0x80A0)
GL_SAMPLE_COVERAGE_INVERT=_C('GL_SAMPLE_COVERAGE_INVERT',0x80AB)
GL_SAMPLE_COVERAGE_VALUE=_C('GL_SAMPLE_COVERAGE_VALUE',0x80AA)
GL_SCISSOR_BOX=_C('GL_SCISSOR_BOX',0x0C10)
GL_SCISSOR_TEST=_C('GL_SCISSOR_TEST',0x0C11)
GL_SHADING_LANGUAGE_VERSION=_C('GL_SHADING_LANGUAGE_VERSION',0x8B8C)
GL_SHORT=_C('GL_SHORT',0x1402)
GL_SRC_ALPHA=_C('GL_SRC_ALPHA',0x0302)
GL_SRC_ALPHA_SATURATE=_C('GL_SRC_ALPHA_SATURATE',0x0308)
GL_SRC_COLOR=_C('GL_SRC_COLOR',0x0300)
GL_STATIC_DRAW=_C('GL_STATIC_DRAW',0x88E4)
GL_STENCIL_ATTACHMENT=_C('GL_STENCIL_ATTACHMENT',0x8D20)
GL_STENCIL_BACK_FAIL=_C('GL_STENCIL_BACK_FAIL',0x8801)
GL_STENCIL_BACK_FUNC=_C('GL_STENCIL_BACK_FUNC',0x8800)
GL_STENCIL_BACK_PASS_DEPTH_FAIL=_C('GL_STENCIL_BACK_PASS_DEPTH_FAIL',0x8802)
GL_STENCIL_BACK_PASS_DEPTH_PASS=_C('GL_STENCIL_BACK_PASS_DEPTH_PASS',0x8803)
GL_STENCIL_BACK_REF=_C('GL_STENCIL_BACK_REF',0x8CA3)
GL_STENCIL_BACK_VALUE_MASK=_C('GL_STENCIL_BACK_VALUE_MASK',0x8CA4)
GL_STENCIL_BACK_WRITEMASK=_C('GL_STENCIL_BACK_WRITEMASK',0x8CA5)
GL_STENCIL_BITS=_C('GL_STENCIL_BITS',0x0D57)
GL_STENCIL_BUFFER_BIT=_C('GL_STENCIL_BUFFER_BIT',0x00000400)
GL_STENCIL_CLEAR_VALUE=_C('GL_STENCIL_CLEAR_VALUE',0x0B91)
GL_STENCIL_FAIL=_C('GL_STENCIL_FAIL',0x0B94)
GL_STENCIL_FUNC=_C('GL_STENCIL_FUNC',0x0B92)
GL_STENCIL_INDEX8=_C('GL_STENCIL_INDEX8',0x8D48)
GL_STENCIL_PASS_DEPTH_FAIL=_C('GL_STENCIL_PASS_DEPTH_FAIL',0x0B95)
GL_STENCIL_PASS_DEPTH_PASS=_C('GL_STENCIL_PASS_DEPTH_PASS',0x0B96)
GL_STENCIL_REF=_C('GL_STENCIL_REF',0x0B97)
GL_STENCIL_TEST=_C('GL_STENCIL_TEST',0x0B90)
GL_STENCIL_VALUE_MASK=_C('GL_STENCIL_VALUE_MASK',0x0B93)
GL_STENCIL_WRITEMASK=_C('GL_STENCIL_WRITEMASK',0x0B98)
GL_STREAM_DRAW=_C('GL_STREAM_DRAW',0x88E0)
GL_SUBPIXEL_BITS=_C('GL_SUBPIXEL_BITS',0x0D50)
GL_TEXTURE=_C('GL_TEXTURE',0x1702)
GL_TEXTURE0=_C('GL_TEXTURE0',0x84C0)
GL_TEXTURE1=_C('GL_TEXTURE1',0x84C1)
GL_TEXTURE10=_C('GL_TEXTURE10',0x84CA)
GL_TEXTURE11=_C('GL_TEXTURE11',0x84CB)
GL_TEXTURE12=_C('GL_TEXTURE12',0x84CC)
GL_TEXTURE13=_C('GL_TEXTURE13',0x84CD)
GL_TEXTURE14=_C('GL_TEXTURE14',0x84CE)
GL_TEXTURE15=_C('GL_TEXTURE15',0x84CF)
GL_TEXTURE16=_C('GL_TEXTURE16',0x84D0)
GL_TEXTURE17=_C('GL_TEXTURE17',0x84D1)
GL_TEXTURE18=_C('GL_TEXTURE18',0x84D2)
GL_TEXTURE19=_C('GL_TEXTURE19',0x84D3)
GL_TEXTURE2=_C('GL_TEXTURE2',0x84C2)
GL_TEXTURE20=_C('GL_TEXTURE20',0x84D4)
GL_TEXTURE21=_C('GL_TEXTURE21',0x84D5)
GL_TEXTURE22=_C('GL_TEXTURE22',0x84D6)
GL_TEXTURE23=_C('GL_TEXTURE23',0x84D7)
GL_TEXTURE24=_C('GL_TEXTURE24',0x84D8)
GL_TEXTURE25=_C('GL_TEXTURE25',0x84D9)
GL_TEXTURE26=_C('GL_TEXTURE26',0x84DA)
GL_TEXTURE27=_C('GL_TEXTURE27',0x84DB)
GL_TEXTURE28=_C('GL_TEXTURE28',0x84DC)
GL_TEXTURE29=_C('GL_TEXTURE29',0x84DD)
GL_TEXTURE3=_C('GL_TEXTURE3',0x84C3)
GL_TEXTURE30=_C('GL_TEXTURE30',0x84DE)
GL_TEXTURE31=_C('GL_TEXTURE31',0x84DF)
GL_TEXTURE4=_C('GL_TEXTURE4',0x84C4)
GL_TEXTURE5=_C('GL_TEXTURE5',0x84C5)
GL_TEXTURE6=_C('GL_TEXTURE6',0x84C6)
GL_TEXTURE7=_C('GL_TEXTURE7',0x84C7)
GL_TEXTURE8=_C('GL_TEXTURE8',0x84C8)
GL_TEXTURE9=_C('GL_TEXTURE9',0x84C9)
GL_TEXTURE_2D=_C('GL_TEXTURE_2D',0x0DE1)
GL_TEXTURE_BINDING_2D=_C('GL_TEXTURE_BINDING_2D',0x8069)
GL_TEXTURE_IMMUTABLE_FORMAT=_C('GL_TEXTURE_IMMUTABLE_FORMAT',0x912F)
GL_TEXTURE_MAG_FILTER=_C('GL_TEXTURE_MAG_FILTER',0x2800)
GL_TEXTURE_MIN_FILTER=_C('GL_TEXTURE_MIN_FILTER',0x2801)
GL_TEXTURE_WRAP_S=_C('GL_TEXTURE_WRAP_S',0x2802)
GL_TEXTURE_WRAP_T=_C('GL_TEXTURE_WRAP_T',0x2803)
GL_TRIANGLES=_C('GL_TRIANGLES',0x0004)
GL_TRIANGLE_FAN=_C('GL_TRIANGLE_FAN',0x0006)
GL_TRIANGLE_STRIP=_C('GL_TRIANGLE_STRIP',0x0005)
GL_TRUE=_C('GL_TRUE',1)
GL_UNKNOWN_CONTEXT_RESET=_C('GL_UNKNOWN_CONTEXT_RESET',0x8255)
GL_UNPACK_ALIGNMENT=_C('GL_UNPACK_ALIGNMENT',0x0CF5)
GL_UNSIGNED_BYTE=_C('GL_UNSIGNED_BYTE',0x1401)
GL_UNSIGNED_INT=_C('GL_UNSIGNED_INT',0x1405)
GL_UNSIGNED_SHORT=_C('GL_UNSIGNED_SHORT',0x1403)
GL_UNSIGNED_SHORT_4_4_4_4=_C('GL_UNSIGNED_SHORT_4_4_4_4',0x8033)
GL_UNSIGNED_SHORT_5_5_5_1=_C('GL_UNSIGNED_SHORT_5_5_5_1',0x8034)
GL_UNSIGNED_SHORT_5_6_5=_C('GL_UNSIGNED_SHORT_5_6_5',0x8363)
GL_VENDOR=_C('GL_VENDOR',0x1F00)
GL_VERSION=_C('GL_VERSION',0x1F02)
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING=_C('GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING',0x889F)
GL_VERTEX_ATTRIB_ARRAY_ENABLED=_C('GL_VERTEX_ATTRIB_ARRAY_ENABLED',0x8622)
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED=_C('GL_VERTEX_ATTRIB_ARRAY_NORMALIZED',0x886A)
GL_VERTEX_ATTRIB_ARRAY_POINTER=_C('GL_VERTEX_ATTRIB_ARRAY_POINTER',0x8645)
GL_VERTEX_ATTRIB_ARRAY_SIZE=_C('GL_VERTEX_ATTRIB_ARRAY_SIZE',0x8623)
GL_VERTEX_ATTRIB_ARRAY_STRIDE=_C('GL_VERTEX_ATTRIB_ARRAY_STRIDE',0x8624)
GL_VERTEX_ATTRIB_ARRAY_TYPE=_C('GL_VERTEX_ATTRIB_ARRAY_TYPE',0x8625)
GL_VIEWPORT=_C('GL_VIEWPORT',0x0BA2)
GL_ZERO=_C('GL_ZERO',0)
@_f
@_p.types(None,_cs.GLenum)
def glActiveTexture(texture):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindBuffer(target,buffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindFramebuffer(target,framebuffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindRenderbuffer(target,renderbuffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindTexture(target,texture):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glBlendColor(red,green,blue,alpha):pass
@_f
@_p.types(None,_cs.GLenum)
def glBlendEquation(mode):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glBlendEquationSeparate(modeRGB,modeAlpha):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glBlendFunc(sfactor,dfactor):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLenum)
def glBlendFuncSeparate(sfactorRGB,dfactorRGB,sfactorAlpha,dfactorAlpha):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizeiptr,ctypes.c_void_p,_cs.GLenum)
def glBufferData(target,size,data,usage):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLintptr,_cs.GLsizeiptr,ctypes.c_void_p)
def glBufferSubData(target,offset,size,data):pass
@_f
@_p.types(_cs.GLenum,_cs.GLenum)
def glCheckFramebufferStatus(target):pass
@_f
@_p.types(None,_cs.GLbitfield)
def glClear(mask):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glClearColor(red,green,blue,alpha):pass
@_f
@_p.types(None,_cs.GLfloat)
def glClearDepthf(d):pass
@_f
@_p.types(None,_cs.GLint)
def glClearStencil(s):pass
@_f
@_p.types(None,_cs.GLboolean,_cs.GLboolean,_cs.GLboolean,_cs.GLboolean)
def glColorMask(red,green,blue,alpha):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexSubImage2D(target,level,xoffset,yoffset,width,height,format,imageSize,data):pass
@_f
@_p.types(_cs.GLuint,)
def glCreateProgram():pass
@_f
@_p.types(None,_cs.GLenum)
def glCullFace(mode):pass
@_f
@_p.types(None,_cs.GLenum)
def glDepthFunc(func):pass
@_f
@_p.types(None,_cs.GLboolean)
def glDepthMask(flag):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat)
def glDepthRangef(n,f):pass
@_f
@_p.types(None,_cs.GLenum)
def glDisable(cap):pass
@_f
@_p.types(None,_cs.GLuint)
def glDisableVertexAttribArray(index):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLsizei)
def glDrawArrays(mode,first,count):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p)
def glDrawRangeElements(mode,start,end,count,type,indices):pass
@_f
@_p.types(None,_cs.GLenum)
def glEnable(cap):pass
@_f
@_p.types(None,_cs.GLuint)
def glEnableVertexAttribArray(index):pass
@_f
@_p.types(None,)
def glFinish():pass
@_f
@_p.types(None,)
def glFlush():pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glFramebufferRenderbuffer(target,attachment,renderbuffertarget,renderbuffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint)
def glFramebufferTexture2D(target,attachment,textarget,texture,level):pass
@_f
@_p.types(None,_cs.GLenum)
def glFrontFace(mode):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenBuffers(n,buffers):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenFramebuffers(n,framebuffers):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenRenderbuffers(n,renderbuffers):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenTextures(n,textures):pass
@_f
@_p.types(None,_cs.GLenum)
def glGenerateMipmap(target):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,arrays.GLcharArray)
def glGetAttribLocation(program,name):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLbooleanArray)
def glGetBooleanv(pname,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetBufferParameteriv(target,pname,params):pass
@_f
@_p.types(_cs.GLenum,)
def glGetError():pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glGetFloatv(pname,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetFramebufferAttachmentParameteriv(target,attachment,pname,params):pass
@_f
@_p.types(_cs.GLenum,)
def glGetGraphicsResetStatus():pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glGetIntegerv(pname,data):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetProgramiv(program,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetRenderbufferParameteriv(target,pname,params):pass
@_f
@_p.types(arrays.GLubyteArray,_cs.GLenum)
def glGetString(name):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetTexParameterfv(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetTexParameteriv(target,pname,params):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,arrays.GLcharArray)
def glGetUniformLocation(program,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLvoidpArray)
def glGetVertexAttribPointerv(index,pname,pointer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLfloatArray)
def glGetVertexAttribfv(index,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetVertexAttribiv(index,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glGetnUniformfv(program,location,bufSize,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glGetnUniformiv(program,location,bufSize,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glHint(target,mode):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLenum)
def glIsEnabled(cap):pass
@_f
@_p.types(None,_cs.GLfloat)
def glLineWidth(width):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glPixelStorei(pname,param):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat)
def glPolygonOffset(factor,units):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei)
def glProgramBinary(program,binaryFormat,binary,length):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glReadnPixels(x,y,width,height,format,type,bufSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glRenderbufferStorage(target,internalformat,width,height):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLboolean)
def glSampleCoverage(value,invert):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glScissor(x,y,width,height):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLuint)
def glStencilFunc(func,ref,mask):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLuint)
def glStencilFuncSeparate(face,func,ref,mask):pass
@_f
@_p.types(None,_cs.GLuint)
def glStencilMask(mask):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glStencilMaskSeparate(face,mask):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum)
def glStencilOp(fail,zfail,zpass):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLenum)
def glStencilOpSeparate(face,sfail,dpfail,dppass):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLfloat)
def glTexParameterf(target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glTexParameterfv(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glTexParameteri(target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glTexParameteriv(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glTexStorage2D(target,levels,internalformat,width,height):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTexSubImage2D(target,level,xoffset,yoffset,width,height,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLfloat)
def glUniform1f(location,v0):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glUniform1fv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint)
def glUniform1i(location,v0):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glUniform1iv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLfloat,_cs.GLfloat)
def glUniform2f(location,v0,v1):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glUniform2fv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint)
def glUniform2i(location,v0,v1):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glUniform2iv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glUniform3f(location,v0,v1,v2):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glUniform3fv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glUniform3i(location,v0,v1,v2):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glUniform3iv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glUniform4f(location,v0,v1,v2,v3):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLfloatArray)
def glUniform4fv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glUniform4i(location,v0,v1,v2,v3):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLintArray)
def glUniform4iv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glUniformMatrix2fv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glUniformMatrix3fv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLfloatArray)
def glUniformMatrix4fv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLuint)
def glUseProgram(program):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLfloat)
def glVertexAttrib1f(index,x):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLfloatArray)
def glVertexAttrib1fv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLfloat,_cs.GLfloat)
def glVertexAttrib2f(index,x,y):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLfloatArray)
def glVertexAttrib2fv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glVertexAttrib3f(index,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLfloatArray)
def glVertexAttrib3fv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glVertexAttrib4f(index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLfloatArray)
def glVertexAttrib4fv(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLboolean,_cs.GLsizei,ctypes.c_void_p)
def glVertexAttribPointer(index,size,type,normalized,stride,pointer):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glViewport(x,y,width,height):pass



'''OpenGL extension NV.shader_texture_footprint

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.shader_texture_footprint to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds OpenGL API support for the OpenGL Shading Language
	(GLSL) extension "NV_shader_texture_footprint".  That extension adds a new
	set of texture query functions ("textureFootprint*NV") to GLSL.  These
	built-in functions prepare to perform a filtered texture lookup based on
	coordinates and other parameters passed in by the calling code.  However,
	instead of returning data from the provided texture image, these query
	functions instead return data identifying the _texture footprint_ for an
	equivalent texture access.  The texture footprint identifies a set of
	texels that may be accessed in order to return a filtered result for the
	texture access.
	
	The footprint itself is a structure that includes integer values that
	identify a small neighborhood of texels in the texture being accessed and
	a bitfield that indicates which texels in that neighborhood would be used.
	Each bit in the returned bitfield identifies whether any texel in a small
	aligned block of texels would be fetched by the texture lookup.  The size
	of each block is specified by an access _granularity_ provided by the
	shader.  The minimum granularity supported by this extension is 2x2 (for
	2D textures) and 2x2x2 (for 3D textures); the maximum granularity is
	256x256 (for 2D textures) or 64x32x32 (for 3D textures).  Each footprint
	query returns the footprint from a single texture level.  When using
	minification filters that combine accesses from multiple mipmap levels,
	shaders must perform separate queries for the two levels accessed ("fine"
	and "coarse").  The footprint query also returns a flag indicating if the
	texture lookup would access texels from only one mipmap level or from two
	neighboring levels.
	
	This extension should be useful for multi-pass rendering operations that
	do an initial expensive rendering pass to produce a first image that is
	then used as a texture for a second pass.  If the second pass ends up
	accessing only portions of the first image (e.g., due to visibility), the
	work spent rendering the non-accessed portion of the first image was
	wasted.  With this feature, an application can limit this waste using an
	initial pass over the geometry in the second image that performs a
	footprint query for each visible pixel to determine the set of pixels that
	it needs from the first image.  This pass would accumulate an aggregate
	footprint of all visible pixels into a separate "footprint texture" using
	shader atomics.  Then, when rendering the first image, the application can
	kill all shading work for pixels not in this aggregate footprint.
	
	The implementation of this extension has a number of limitations.  The
	texture footprint query functions are only supported for two- and
	three-dimensional textures (TEXTURE_2D, TEXTURE_3D).  Texture footprint
	evaluation only supports the CLAMP_TO_EDGE wrap mode; results are
	undefined for all other wrap modes.  The implementation supports only a
	limited set of granularity values and does not support separate coverage
	information for each texel in the original texture.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/shader_texture_footprint.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.shader_texture_footprint import *
from OpenGL.raw.GLES2.NV.shader_texture_footprint import _EXTENSION_NAME

def glInitShaderTextureFootprintNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.shadow_samplers_cube

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.shadow_samplers_cube to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension expands the shadow map capability described in
	EXT_shadow_samplers to include support for shadow samplers of cube
	map textures.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/shadow_samplers_cube.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.shadow_samplers_cube import *
from OpenGL.raw.GLES2.NV.shadow_samplers_cube import _EXTENSION_NAME

def glInitShadowSamplersCubeNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.compressed_paletted_texture

This module customises the behaviour of the 
OpenGL.raw.GL.OES.compressed_paletted_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	The goal of this extension is to allow direct support of palettized
	textures in OpenGL ES.
	
	Palettized textures are implemented in OpenGL ES using the
	CompressedTexImage2D call. The definition of the following parameters
	"level" and "internalformat" in the CompressedTexImage2D call have
	been extended to support paletted textures.
	
	A paletted texture is described by the following data:
	
	    palette format
	        can be R5_G6_B5, RGBA4, RGB5_A1, RGB8, or RGBA8
	
	    number of bits to represent texture data
	        can be 4 bits or 8 bits per texel.  The number of bits
	        also detemine the size of the palette.  For 4 bits/texel
	        the palette size is 16 entries and for 8 bits/texel the
	        palette size will be 256 entries.
	
	        The palette format and bits/texel are encoded in the
	        "internalformat" parameter.
	
	    palette data and texture mip-levels
	        The palette data followed by all necessary mip levels are
	        passed in "data" parameter of CompressedTexImage2D.
	
	        The size of palette is given by palette format and bits / texel.
	        A palette format of RGB_565 with 4 bits/texel imply a palette
	        size of 2 bytes/palette entry * 16 entries = 32 bytes.
	
	        The level value is used to indicate how many mip levels
	        are described.  Negative level values are used to define
	        the number of miplevels described in the "data" component.
	        A level of zero indicates a single mip-level.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/compressed_paletted_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.OES.compressed_paletted_texture import *
from OpenGL.raw.GL.OES.compressed_paletted_texture import _EXTENSION_NAME

def glInitCompressedPalettedTextureOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGI_color_table'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGI_color_table',error_checker=_errors._error_checker)
GL_COLOR_TABLE_ALPHA_SIZE_SGI=_C('GL_COLOR_TABLE_ALPHA_SIZE_SGI',0x80DD)
GL_COLOR_TABLE_BIAS_SGI=_C('GL_COLOR_TABLE_BIAS_SGI',0x80D7)
GL_COLOR_TABLE_BLUE_SIZE_SGI=_C('GL_COLOR_TABLE_BLUE_SIZE_SGI',0x80DC)
GL_COLOR_TABLE_FORMAT_SGI=_C('GL_COLOR_TABLE_FORMAT_SGI',0x80D8)
GL_COLOR_TABLE_GREEN_SIZE_SGI=_C('GL_COLOR_TABLE_GREEN_SIZE_SGI',0x80DB)
GL_COLOR_TABLE_INTENSITY_SIZE_SGI=_C('GL_COLOR_TABLE_INTENSITY_SIZE_SGI',0x80DF)
GL_COLOR_TABLE_LUMINANCE_SIZE_SGI=_C('GL_COLOR_TABLE_LUMINANCE_SIZE_SGI',0x80DE)
GL_COLOR_TABLE_RED_SIZE_SGI=_C('GL_COLOR_TABLE_RED_SIZE_SGI',0x80DA)
GL_COLOR_TABLE_SCALE_SGI=_C('GL_COLOR_TABLE_SCALE_SGI',0x80D6)
GL_COLOR_TABLE_SGI=_C('GL_COLOR_TABLE_SGI',0x80D0)
GL_COLOR_TABLE_WIDTH_SGI=_C('GL_COLOR_TABLE_WIDTH_SGI',0x80D9)
GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI=_C('GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI',0x80D2)
GL_POST_CONVOLUTION_COLOR_TABLE_SGI=_C('GL_POST_CONVOLUTION_COLOR_TABLE_SGI',0x80D1)
GL_PROXY_COLOR_TABLE_SGI=_C('GL_PROXY_COLOR_TABLE_SGI',0x80D3)
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI=_C('GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI',0x80D5)
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI=_C('GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI',0x80D4)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glColorTableParameterfvSGI(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glColorTableParameterivSGI(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glColorTableSGI(target,internalformat,width,format,type,table):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei)
def glCopyColorTableSGI(target,internalformat,x,y,width):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetColorTableParameterfvSGI(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetColorTableParameterivSGI(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glGetColorTableSGI(target,format,type,table):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLX import _types as _cs
# End users want this...
from OpenGL.raw.GLX._types import *
from OpenGL.raw.GLX import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLX_SGI_cushion'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLX,'GLX_SGI_cushion',error_checker=_errors._error_checker)

@_f
@_p.types(None,ctypes.POINTER(_cs.Display),_cs.Window,ctypes.c_float)
def glXCushionSGI(dpy,window,cushion):pass



'''OpenGL extension ARB.gpu_shader_int64

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.gpu_shader_int64 to provide a more 
Python-friendly API

Overview (from the spec)
	
	The extension introduces the following features for all shader types:
	
	  * support for 64-bit scalar and vector integer data types, including
	    uniform API, uniform buffer object, transform feedback, and shader
	    input and output support;
	
	  * new built-in functions to pack and unpack 64-bit integer types into a
	    two-component 32-bit integer vector;
	
	  * new built-in functions to convert double-precision floating-point
	    values to or from their 64-bit integer bit encodings;
	
	  * vector relational functions supporting comparisons of vectors of
	    64-bit integer types; and
	
	  * common functions abs, sign, min, max, clamp, and mix supporting
	    arguments of 64-bit integer types.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/gpu_shader_int64.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.gpu_shader_int64 import *
from OpenGL.raw.GL.ARB.gpu_shader_int64 import _EXTENSION_NAME

def glInitGpuShaderInt64ARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glUniform1i64vARB.value size not checked against count
glUniform1i64vARB=wrapper.wrapper(glUniform1i64vARB).setInputArraySize(
    'value', None
)
# INPUT glUniform2i64vARB.value size not checked against count*2
glUniform2i64vARB=wrapper.wrapper(glUniform2i64vARB).setInputArraySize(
    'value', None
)
# INPUT glUniform3i64vARB.value size not checked against count*3
glUniform3i64vARB=wrapper.wrapper(glUniform3i64vARB).setInputArraySize(
    'value', None
)
# INPUT glUniform4i64vARB.value size not checked against count*4
glUniform4i64vARB=wrapper.wrapper(glUniform4i64vARB).setInputArraySize(
    'value', None
)
# INPUT glUniform1ui64vARB.value size not checked against count
glUniform1ui64vARB=wrapper.wrapper(glUniform1ui64vARB).setInputArraySize(
    'value', None
)
# INPUT glUniform2ui64vARB.value size not checked against count*2
glUniform2ui64vARB=wrapper.wrapper(glUniform2ui64vARB).setInputArraySize(
    'value', None
)
# INPUT glUniform3ui64vARB.value size not checked against count*3
glUniform3ui64vARB=wrapper.wrapper(glUniform3ui64vARB).setInputArraySize(
    'value', None
)
# INPUT glUniform4ui64vARB.value size not checked against count*4
glUniform4ui64vARB=wrapper.wrapper(glUniform4ui64vARB).setInputArraySize(
    'value', None
)
# INPUT glGetUniformi64vARB.params size not checked against 'program,location'
glGetUniformi64vARB=wrapper.wrapper(glGetUniformi64vARB).setInputArraySize(
    'params', None
)
# INPUT glGetUniformui64vARB.params size not checked against 'program,location'
glGetUniformui64vARB=wrapper.wrapper(glGetUniformui64vARB).setInputArraySize(
    'params', None
)
# INPUT glGetnUniformi64vARB.params size not checked against bufSize
glGetnUniformi64vARB=wrapper.wrapper(glGetnUniformi64vARB).setInputArraySize(
    'params', None
)
# INPUT glGetnUniformui64vARB.params size not checked against bufSize
glGetnUniformui64vARB=wrapper.wrapper(glGetnUniformui64vARB).setInputArraySize(
    'params', None
)
# INPUT glProgramUniform1i64vARB.value size not checked against count
glProgramUniform1i64vARB=wrapper.wrapper(glProgramUniform1i64vARB).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2i64vARB.value size not checked against count*2
glProgramUniform2i64vARB=wrapper.wrapper(glProgramUniform2i64vARB).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3i64vARB.value size not checked against count*3
glProgramUniform3i64vARB=wrapper.wrapper(glProgramUniform3i64vARB).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4i64vARB.value size not checked against count*4
glProgramUniform4i64vARB=wrapper.wrapper(glProgramUniform4i64vARB).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform1ui64vARB.value size not checked against count
glProgramUniform1ui64vARB=wrapper.wrapper(glProgramUniform1ui64vARB).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2ui64vARB.value size not checked against count*2
glProgramUniform2ui64vARB=wrapper.wrapper(glProgramUniform2ui64vARB).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3ui64vARB.value size not checked against count*3
glProgramUniform3ui64vARB=wrapper.wrapper(glProgramUniform3ui64vARB).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4ui64vARB.value size not checked against count*4
glProgramUniform4ui64vARB=wrapper.wrapper(glProgramUniform4ui64vARB).setInputArraySize(
    'value', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension VERSION.GLES2_2_0

This module customises the behaviour of the 
OpenGL.raw.GLES2.VERSION.GLES2_2_0 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GLES2_2_0.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.VERSION.GLES2_2_0 import *
from OpenGL.raw.GLES2.VERSION.GLES2_2_0 import _EXTENSION_NAME

def glInitGles220VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glBufferData.data size not checked against size
glBufferData=wrapper.wrapper(glBufferData).setInputArraySize(
    'data', None
)
# INPUT glBufferSubData.data size not checked against size
glBufferSubData=wrapper.wrapper(glBufferSubData).setInputArraySize(
    'data', None
)
# INPUT glCompressedTexImage2D.data size not checked against imageSize
glCompressedTexImage2D=wrapper.wrapper(glCompressedTexImage2D).setInputArraySize(
    'data', None
)
# INPUT glCompressedTexSubImage2D.data size not checked against imageSize
glCompressedTexSubImage2D=wrapper.wrapper(glCompressedTexSubImage2D).setInputArraySize(
    'data', None
)
# INPUT glDeleteBuffers.buffers size not checked against n
glDeleteBuffers=wrapper.wrapper(glDeleteBuffers).setInputArraySize(
    'buffers', None
)
# INPUT glDeleteFramebuffers.framebuffers size not checked against n
glDeleteFramebuffers=wrapper.wrapper(glDeleteFramebuffers).setInputArraySize(
    'framebuffers', None
)
# INPUT glDeleteRenderbuffers.renderbuffers size not checked against n
glDeleteRenderbuffers=wrapper.wrapper(glDeleteRenderbuffers).setInputArraySize(
    'renderbuffers', None
)
# INPUT glDeleteTextures.textures size not checked against n
glDeleteTextures=wrapper.wrapper(glDeleteTextures).setInputArraySize(
    'textures', None
)
# INPUT glDrawElements.indices size not checked against 'count,type'
glDrawElements=wrapper.wrapper(glDrawElements).setInputArraySize(
    'indices', None
)
glGenBuffers=wrapper.wrapper(glGenBuffers).setOutput(
    'buffers',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGenFramebuffers=wrapper.wrapper(glGenFramebuffers).setOutput(
    'framebuffers',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGenRenderbuffers=wrapper.wrapper(glGenRenderbuffers).setOutput(
    'renderbuffers',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGenTextures=wrapper.wrapper(glGenTextures).setOutput(
    'textures',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGetActiveAttrib=wrapper.wrapper(glGetActiveAttrib).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'name',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'size',size=(1,),orPassIn=True
).setOutput(
    'type',size=(1,),orPassIn=True
)
glGetActiveUniform=wrapper.wrapper(glGetActiveUniform).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'name',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'size',size=(1,),orPassIn=True
).setOutput(
    'type',size=(1,),orPassIn=True
)
# glGetAttachedShaders.obj is OUTPUT without known output size
# INPUT glGetAttachedShaders.shaders size not checked against maxCount
glGetAttachedShaders=wrapper.wrapper(glGetAttachedShaders).setOutput(
    'count',size=(1,),orPassIn=True
).setInputArraySize(
    'shaders', None
)
glGetBooleanv=wrapper.wrapper(glGetBooleanv).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetBufferParameteriv=wrapper.wrapper(glGetBufferParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetFloatv=wrapper.wrapper(glGetFloatv).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetFramebufferAttachmentParameteriv=wrapper.wrapper(glGetFramebufferAttachmentParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetIntegerv=wrapper.wrapper(glGetIntegerv).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetProgramiv=wrapper.wrapper(glGetProgramiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetProgramInfoLog=wrapper.wrapper(glGetProgramInfoLog).setOutput(
    'infoLog',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'length',size=(1,),orPassIn=True
)
glGetRenderbufferParameteriv=wrapper.wrapper(glGetRenderbufferParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetShaderiv=wrapper.wrapper(glGetShaderiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetShaderInfoLog=wrapper.wrapper(glGetShaderInfoLog).setOutput(
    'infoLog',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'length',size=(1,),orPassIn=True
)
glGetShaderPrecisionFormat=wrapper.wrapper(glGetShaderPrecisionFormat).setOutput(
    'precision',size=(1,),orPassIn=True
).setOutput(
    'range',size=(2,),orPassIn=True
)
glGetShaderSource=wrapper.wrapper(glGetShaderSource).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'source',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetTexParameterfv=wrapper.wrapper(glGetTexParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTexParameteriv=wrapper.wrapper(glGetTexParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# glGetUniformfv.params is OUTPUT without known output size
# glGetUniformiv.params is OUTPUT without known output size
glGetVertexAttribfv=wrapper.wrapper(glGetVertexAttribfv).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribiv=wrapper.wrapper(glGetVertexAttribiv).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetVertexAttribPointerv=wrapper.wrapper(glGetVertexAttribPointerv).setOutput(
    'pointer',size=(1,),orPassIn=True
)
# OUTPUT glReadPixels.pixels COMPSIZE(format, type, width, height) 
# INPUT glShaderBinary.binary size not checked against length
# INPUT glShaderBinary.shaders size not checked against count
glShaderBinary=wrapper.wrapper(glShaderBinary).setInputArraySize(
    'binary', None
).setInputArraySize(
    'shaders', None
)
# INPUT glShaderSource.length size not checked against count
# INPUT glShaderSource.string size not checked against count
glShaderSource=wrapper.wrapper(glShaderSource).setInputArraySize(
    'length', None
).setInputArraySize(
    'string', None
)
# INPUT glTexImage2D.pixels size not checked against 'format,type,width,height'
glTexImage2D=wrapper.wrapper(glTexImage2D).setInputArraySize(
    'pixels', None
)
# INPUT glTexParameterfv.params size not checked against 'pname'
glTexParameterfv=wrapper.wrapper(glTexParameterfv).setInputArraySize(
    'params', None
)
# INPUT glTexParameteriv.params size not checked against 'pname'
glTexParameteriv=wrapper.wrapper(glTexParameteriv).setInputArraySize(
    'params', None
)
# INPUT glTexSubImage2D.pixels size not checked against 'format,type,width,height'
glTexSubImage2D=wrapper.wrapper(glTexSubImage2D).setInputArraySize(
    'pixels', None
)
# INPUT glUniform1fv.value size not checked against count
glUniform1fv=wrapper.wrapper(glUniform1fv).setInputArraySize(
    'value', None
)
# INPUT glUniform1iv.value size not checked against count
glUniform1iv=wrapper.wrapper(glUniform1iv).setInputArraySize(
    'value', None
)
# INPUT glUniform2fv.value size not checked against count*2
glUniform2fv=wrapper.wrapper(glUniform2fv).setInputArraySize(
    'value', None
)
# INPUT glUniform2iv.value size not checked against count*2
glUniform2iv=wrapper.wrapper(glUniform2iv).setInputArraySize(
    'value', None
)
# INPUT glUniform3fv.value size not checked against count*3
glUniform3fv=wrapper.wrapper(glUniform3fv).setInputArraySize(
    'value', None
)
# INPUT glUniform3iv.value size not checked against count*3
glUniform3iv=wrapper.wrapper(glUniform3iv).setInputArraySize(
    'value', None
)
# INPUT glUniform4fv.value size not checked against count*4
glUniform4fv=wrapper.wrapper(glUniform4fv).setInputArraySize(
    'value', None
)
# INPUT glUniform4iv.value size not checked against count*4
glUniform4iv=wrapper.wrapper(glUniform4iv).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix2fv.value size not checked against count*4
glUniformMatrix2fv=wrapper.wrapper(glUniformMatrix2fv).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix3fv.value size not checked against count*9
glUniformMatrix3fv=wrapper.wrapper(glUniformMatrix3fv).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix4fv.value size not checked against count*16
glUniformMatrix4fv=wrapper.wrapper(glUniformMatrix4fv).setInputArraySize(
    'value', None
)
glVertexAttrib1fv=wrapper.wrapper(glVertexAttrib1fv).setInputArraySize(
    'v', 1
)
glVertexAttrib2fv=wrapper.wrapper(glVertexAttrib2fv).setInputArraySize(
    'v', 2
)
glVertexAttrib3fv=wrapper.wrapper(glVertexAttrib3fv).setInputArraySize(
    'v', 3
)
glVertexAttrib4fv=wrapper.wrapper(glVertexAttrib4fv).setInputArraySize(
    'v', 4
)
# INPUT glVertexAttribPointer.pointer size not checked against 'size,type,stride'
glVertexAttribPointer=wrapper.wrapper(glVertexAttribPointer).setInputArraySize(
    'pointer', None
)
### END AUTOGENERATED SECTION
from OpenGL import converters
from OpenGL.lazywrapper import lazy as _lazy
from OpenGL._bytes import _NULL_8_BYTE
from OpenGL import contextdata 

glShaderSource = platform.createExtensionFunction(
    'glShaderSource', dll=platform.PLATFORM.GLES2,
    resultType=None,
    argTypes=(_types.GLhandle, _types.GLsizei, ctypes.POINTER(ctypes.c_char_p), arrays.GLintArray,),
    doc = 'glShaderSource( GLhandle(shaderObj),[bytes(string),...]) -> None',
    argNames = ('shaderObj', 'count', 'string', 'length',),
    extension = _EXTENSION_NAME,
)

conv = converters.StringLengths( name='string' )
glShaderSource = wrapper.wrapper(
    glShaderSource
).setPyConverter(
    'count' # number of strings
).setPyConverter(
    'length' # lengths of strings
).setPyConverter(
    'string', conv.stringArray
).setCResolver(
    'string', conv.stringArrayForC,
).setCConverter(
    'length', conv,
).setCConverter(
    'count', conv.totalCount,
)
try:
    del conv
except NameError as err:
    pass


@_lazy( glGetShaderInfoLog )
def glGetShaderInfoLog( baseOperation, obj ):
    """Retrieve the shader's error messages as a Python string

    returns string which is '' if no message
    """
    target = GLsizei()
    glGetShaderiv(obj, GL_INFO_LOG_LENGTH,target)
    length = target.value
    if length > 0:
        log = ctypes.create_string_buffer(length)
        baseOperation(obj, length, None, log)
        return log.value.strip(_NULL_8_BYTE) # null-termination
    return ''
@_lazy( glGetProgramInfoLog )
def glGetProgramInfoLog( baseOperation, obj ):
    """Retrieve the shader program's error messages as a Python string

    returns string which is '' if no message
    """
    target = GLsizei()
    glGetProgramiv(obj, GL_INFO_LOG_LENGTH,target)
    length = target.value
    if length > 0:
        log = ctypes.create_string_buffer(length)
        baseOperation(obj, length, None, log)
        return log.value.strip(_NULL_8_BYTE) # null-termination
    return ''

@_lazy( glGetAttachedShaders )
def glGetAttachedShaders( baseOperation, obj ):
    """Retrieve the attached objects as an array of GLhandle instances"""
    length= glGetProgramiv( obj, GL_ATTACHED_SHADERS )
    if length > 0:
        storage = arrays.GLuintArray.zeros( (length,))
        baseOperation( obj, length, None, storage )
        return storage
    return arrays.GLuintArray.zeros( (0,))


@_lazy( glGetShaderSource )
def glGetShaderSource( baseOperation, obj ):
    """Retrieve the program/shader's source code as a Python string

    returns string which is '' if no source code
    """
    length = int(glGetShaderiv(obj, GL_SHADER_SOURCE_LENGTH))
    if length > 0:
        source = ctypes.create_string_buffer(length)
        baseOperation(obj, length, None, source)
        return source.value.strip(_NULL_8_BYTE) # null-termination
    return ''

@_lazy( glGetActiveUniform )
def glGetActiveUniform(baseOperation,program, index,bufSize=None,*args):
    """Retrieve the name, size and type of the uniform of the index in the program
    
    program -- specifies the program to be queried
    index -- index of the uniform to be queried 
    
    Following parameters are optional:
    
    bufSize -- determines the size of the buffer (limits number of bytes written),
               if not provided, will be GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH
    length -- pointer-to-GLsizei that will hold the resulting length of the name
    size -- pointer-to-GLint that will hold the size of the attribute
    type -- pointer-to-GLenum that will hold the type constant of the attribute
    name -- pointer-to-GLchar that will hold the (null-terminated) name string
    
    returns (bytes) name, (int)size, (enum)type
    """
    max_index = int(glGetProgramiv( program, GL_ACTIVE_UNIFORMS ))
    if bufSize is None:
        bufSize = int(glGetProgramiv( program, GL_ACTIVE_UNIFORM_MAX_LENGTH))
    if index < max_index and index >= 0:
        length,name,size,type = baseOperation( program, index, bufSize, *args )
        if hasattr(name,'tostring'):
            name = name.tostring().rstrip(b'\000')
        elif hasattr(name,'value'):
            name = name.value
        return name,size,type
    raise IndexError( 'Index %s out of range 0 to %i' % (index, max_index - 1, ) )

@_lazy( glGetActiveAttrib )
def glGetActiveAttrib(baseOperation, program, index, bufSize=None,*args):
    """Retrieves information about the attribute variable.

    program -- specifies the program to be queried
    index -- index of the attribute to be queried 
    
    Following parameters are optional:
    
    bufSize -- determines the size of the buffer (limits number of bytes written),
               if not provided, will be GL_ACTIVE_ATTRIBUTE_MAX_LENGTH
    length -- pointer-to-GLsizei that will hold the resulting length of the name
    size -- pointer-to-GLint that will hold the size of the attribute
    type -- pointer-to-GLenum that will hold the type constant of the attribute
    name -- pointer-to-GLchar that will hold the (null-terminated) name string
    
    returns (bytes) name, (int)size, (enum)type
    """
    if bufSize is None:
        bufSize = int(glGetProgramiv( program, GL_ACTIVE_ATTRIBUTE_MAX_LENGTH))
    if bufSize <= 0:
        raise RuntimeError( 'Active attribute length reported', bufsize )
    name,size,type = baseOperation( program, index, bufSize, *args )[1:]
    if hasattr(name,'tostring'):
        name = name.tostring().rstrip(b'\000')
    elif hasattr(name,'value'):
        name = name.value
    return name,size,type


@_lazy( glGetUniformLocation )
def glGetUniformLocation( baseOperation, program, name ):
    """Check that name is a string with a null byte at the end of it"""
    if not name:
        raise ValueError( """Non-null name required""" )
    name = as_8_bit( name )
    if name[-1] != _NULL_8_BYTE:
        name = name + _NULL_8_BYTE
    return baseOperation( program, name )
@_lazy( glGetAttribLocation )
def glGetAttribLocation( baseOperation, program, name ):
    """Check that name is a string with a null byte at the end of it"""
    if not name:
        raise ValueError( """Non-null name required""" )
    
    name = as_8_bit( name )
    if name[-1] != _NULL_8_BYTE:
        name = name + _NULL_8_BYTE
    return baseOperation( program, name )

@_lazy( glVertexAttribPointer )
def glVertexAttribPointer(
    baseOperation, index, size, type,
    normalized, stride, pointer,
):
    """Set an attribute pointer for a given shader (index)

    index -- the index of the generic vertex to bind, see
        glGetAttribLocation for retrieval of the value,
        note that index is a global variable, not per-shader
    size -- number of basic elements per record, 1,2,3, or 4
    type -- enum constant for data-type
    normalized -- whether to perform int to float
        normalization on integer-type values
    stride -- stride in machine units (bytes) between
        consecutive records, normally used to create
        "interleaved" arrays
    pointer -- data-pointer which provides the data-values,
        normally a vertex-buffer-object or offset into the
        same.

    This implementation stores a copy of the data-pointer
    in the contextdata structure in order to prevent null-
    reference errors in the renderer.
    """
    array = arrays.ArrayDatatype.asArray( pointer, type )
    key = ('vertex-attrib',index)
    contextdata.setValue( key, array )
    return baseOperation(
        index, size, type,
        normalized, stride,
        arrays.ArrayDatatype.voidDataPointer( array )
    )




'''OpenGL extension EXT.unpack_subimage

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.unpack_subimage to provide a more 
Python-friendly API

Overview (from the spec)
	
	This OpenGL ES 2.0 extension adds support for GL_UNPACK_ROW_LENGTH,
	GL_UNPACK_SKIP_ROWS and GL_UNPACK_SKIP_PIXELS as valid enums to
	PixelStore.  The functionality is the same as in OpenGL. These are
	useful for updating textures with a sub-rectangle of pixel data
	coming from a larger image in host memory.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/unpack_subimage.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.unpack_subimage import *
from OpenGL.raw.GLES2.EXT.unpack_subimage import _EXTENSION_NAME

def glInitUnpackSubimageEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.texture_buffer

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.texture_buffer to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a new texture type, called a buffer texture.
	Buffer textures are one-dimensional arrays of texels whose storage comes
	from an attached buffer object. When a buffer object is bound to a
	buffer texture, a format is specified, and the data in the buffer object
	is treated as an array of texels of the specified format.
	
	The use of a buffer object to provide storage allows the texture data to
	be specified in a number of different ways: via buffer object loads
	(BufferData), direct CPU writes (MapBuffer), or framebuffer readbacks to
	pixel buffer objects (ReadPixels). A buffer object can also be loaded by
	transform feedback, which captures
	selected transformed attributes of vertices processed by the GL. Several
	of these mechanisms do not require an extra data copy, which would be
	required when using conventional TexImage-like entry points.
	
	Buffer textures do not support mipmapping, texture lookups with
	normalized floating-point texture coordinates, and texture filtering of
	any sort.
	They can be accessed via single texel fetch operations in programmable
	shaders, using a new sampler type and texel fetch function, and
	access can be controlled using the same memory barrier operations
	as for other texture types.
	
	Buffer textures are treated as (potentially large) one-dimensional
	textures; the maximum texture size supported for buffer textures in the
	initial implementation of this extension is 2^27 texels (note that this
	extension only guarantees support for buffer textures with 2^16 texels,
	but we expect most implementations to exceed that substantially). When a
	buffer object is attached to a buffer texture, a size is not specified;
	rather, the number of texels in the texture is taken by dividing the size
	of the buffer object by the size of each texel.
	
	This extension also allows a sub-range of the buffer's data store to
	be attached to a texture. This can be used, for example, to allow multiple
	buffer textures to be backed by independent sub-ranges of the same buffer
	object, or for different sub-ranges of a single buffer object to be used
	for different purposes.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/texture_buffer.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.texture_buffer import *
from OpenGL.raw.GLES2.OES.texture_buffer import _EXTENSION_NAME

def glInitTextureBufferOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_KHR_shader_subgroup'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_KHR_shader_subgroup',error_checker=_errors._error_checker)
GL_SUBGROUP_FEATURE_ARITHMETIC_BIT_KHR=_C('GL_SUBGROUP_FEATURE_ARITHMETIC_BIT_KHR',0x00000004)
GL_SUBGROUP_FEATURE_BALLOT_BIT_KHR=_C('GL_SUBGROUP_FEATURE_BALLOT_BIT_KHR',0x00000008)
GL_SUBGROUP_FEATURE_BASIC_BIT_KHR=_C('GL_SUBGROUP_FEATURE_BASIC_BIT_KHR',0x00000001)
GL_SUBGROUP_FEATURE_CLUSTERED_BIT_KHR=_C('GL_SUBGROUP_FEATURE_CLUSTERED_BIT_KHR',0x00000040)
GL_SUBGROUP_FEATURE_QUAD_BIT_KHR=_C('GL_SUBGROUP_FEATURE_QUAD_BIT_KHR',0x00000080)
GL_SUBGROUP_FEATURE_SHUFFLE_BIT_KHR=_C('GL_SUBGROUP_FEATURE_SHUFFLE_BIT_KHR',0x00000010)
GL_SUBGROUP_FEATURE_SHUFFLE_RELATIVE_BIT_KHR=_C('GL_SUBGROUP_FEATURE_SHUFFLE_RELATIVE_BIT_KHR',0x00000020)
GL_SUBGROUP_FEATURE_VOTE_BIT_KHR=_C('GL_SUBGROUP_FEATURE_VOTE_BIT_KHR',0x00000002)
GL_SUBGROUP_QUAD_ALL_STAGES_KHR=_C('GL_SUBGROUP_QUAD_ALL_STAGES_KHR',0x9535)
GL_SUBGROUP_SIZE_KHR=_C('GL_SUBGROUP_SIZE_KHR',0x9532)
GL_SUBGROUP_SUPPORTED_FEATURES_KHR=_C('GL_SUBGROUP_SUPPORTED_FEATURES_KHR',0x9534)
GL_SUBGROUP_SUPPORTED_STAGES_KHR=_C('GL_SUBGROUP_SUPPORTED_STAGES_KHR',0x9533)




'''OpenGL extension EXT.texture_compression_bptc

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_compression_bptc to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides additional texture compression functionality
	specific to the BPTC and BPTC_FLOAT compressed texture formats (called BC7
	and BC6H respectively in Microsoft's DirectX API).
	
	Traditional block compression methods as typified by s3tc and latc
	compress a block of pixels into indicies along a gradient. This works well
	for smooth images, but can have quality issues along sharp edges and
	strong chrominance transitions. To improve quality in these problematic
	cases, the BPTC formats can divide each block into multiple partitions,
	each of which are compressed using an independent gradient.
	
	In addition, it is desirable to directly support high dynamic range
	imagery in compressed formats, which is accomplished by the BPTC_FLOAT
	formats.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_compression_bptc.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_compression_bptc import *
from OpenGL.raw.GLES2.EXT.texture_compression_bptc import _EXTENSION_NAME

def glInitTextureCompressionBptcEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.matrix_get

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.matrix_get to provide a more 
Python-friendly API

Overview (from the spec)
	
	Many applications require the ability to be able to read the
	GL matrices.  OpenGL ES 1.1 will allow an application to read
	the matrices using the GetFloatv command for the common profile
	and the GetFixedv command for the common-lite profile.
	
	In cases where the common-lite implementation stores matrices
	and performs matrix operations internally using floating pt 
	(example would be OpenGL ES implementations that support JSR184 etc.)
	the GL cannot return the floating pt matrix elements since the float
	data type is not supported by the common-lite profile.
	Using GetFixedv to get the matrix data will result in a loss of
	information.
	
	To take care of this issue, new tokens are proposed by this
	extension.  These tokens will allow the GL to return a 
	representation of the floating pt matrix elements as as an array
	of integers, according to the IEEE 754 floating pt "single format"
	bit layout.
	
	Bit 31 represents the sign of the floating pt number.
	Bits 30 - 23 represent the exponent of the floating pt number.
	Bits 22 - 0 represent the mantissa of the floating pt number.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/matrix_get.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.matrix_get import *
from OpenGL.raw.GLES1.OES.matrix_get import _EXTENSION_NAME

def glInitMatrixGetOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.shader_pixel_local_storage2

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.shader_pixel_local_storage2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension builds on EXT_shader_pixel_local_storage by lifting the
	restriction that pixel local storage is not supported when rendering to
	multiple draw buffers.
	
	Moreover, pixel local storage values are no longer lost when writing to
	user-defined fragment outputs, and, correspondingly, framebuffer pixel
	values do not always become undefined when the shader writes to pixel local
	storage.
	
	This extension adds the following capabilities:
	- support for pixel local storage in combination with multiple user-
	  defined fragment outputs
	- support for clearing pixel local storage variables
	- support for multi-word pixel local storage variables

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/shader_pixel_local_storage2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.shader_pixel_local_storage2 import *
from OpenGL.raw.GLES2.EXT.shader_pixel_local_storage2 import _EXTENSION_NAME

def glInitShaderPixelLocalStorage2EXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glClearPixelLocalStorageuiEXT.values size not checked against n
glClearPixelLocalStorageuiEXT=wrapper.wrapper(glClearPixelLocalStorageuiEXT).setInputArraySize(
    'values', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_4_6'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_4_6',error_checker=_errors._error_checker)
GL_CLIPPING_INPUT_PRIMITIVES=_C('GL_CLIPPING_INPUT_PRIMITIVES',0x82F6)
GL_CLIPPING_OUTPUT_PRIMITIVES=_C('GL_CLIPPING_OUTPUT_PRIMITIVES',0x82F7)
GL_COMPUTE_SHADER_INVOCATIONS=_C('GL_COMPUTE_SHADER_INVOCATIONS',0x82F5)
GL_CONTEXT_FLAG_NO_ERROR_BIT=_C('GL_CONTEXT_FLAG_NO_ERROR_BIT',0x00000008)
GL_CONTEXT_RELEASE_BEHAVIOR=_C('GL_CONTEXT_RELEASE_BEHAVIOR',0x82FB)
GL_CONTEXT_RELEASE_BEHAVIOR_FLUSH=_C('GL_CONTEXT_RELEASE_BEHAVIOR_FLUSH',0x82FC)
GL_FRAGMENT_SHADER_INVOCATIONS=_C('GL_FRAGMENT_SHADER_INVOCATIONS',0x82F4)
GL_GEOMETRY_SHADER_INVOCATIONS=_C('GL_GEOMETRY_SHADER_INVOCATIONS',0x887F)
GL_GEOMETRY_SHADER_PRIMITIVES_EMITTED=_C('GL_GEOMETRY_SHADER_PRIMITIVES_EMITTED',0x82F3)
GL_MAX_TEXTURE_MAX_ANISOTROPY=_C('GL_MAX_TEXTURE_MAX_ANISOTROPY',0x84FF)
GL_NONE=_C('GL_NONE',0)
GL_NUM_SPIR_V_EXTENSIONS=_C('GL_NUM_SPIR_V_EXTENSIONS',0x9554)
GL_PARAMETER_BUFFER=_C('GL_PARAMETER_BUFFER',0x80EE)
GL_PARAMETER_BUFFER_BINDING=_C('GL_PARAMETER_BUFFER_BINDING',0x80EF)
GL_POLYGON_OFFSET_CLAMP=_C('GL_POLYGON_OFFSET_CLAMP',0x8E1B)
GL_PRIMITIVES_SUBMITTED=_C('GL_PRIMITIVES_SUBMITTED',0x82EF)
GL_SHADER_BINARY_FORMAT_SPIR_V=_C('GL_SHADER_BINARY_FORMAT_SPIR_V',0x9551)
GL_SPIR_V_BINARY=_C('GL_SPIR_V_BINARY',0x9552)
GL_SPIR_V_EXTENSIONS=_C('GL_SPIR_V_EXTENSIONS',0x9553)
GL_TESS_CONTROL_SHADER_PATCHES=_C('GL_TESS_CONTROL_SHADER_PATCHES',0x82F1)
GL_TESS_EVALUATION_SHADER_INVOCATIONS=_C('GL_TESS_EVALUATION_SHADER_INVOCATIONS',0x82F2)
GL_TEXTURE_MAX_ANISOTROPY=_C('GL_TEXTURE_MAX_ANISOTROPY',0x84FE)
GL_TRANSFORM_FEEDBACK_OVERFLOW=_C('GL_TRANSFORM_FEEDBACK_OVERFLOW',0x82EC)
GL_TRANSFORM_FEEDBACK_STREAM_OVERFLOW=_C('GL_TRANSFORM_FEEDBACK_STREAM_OVERFLOW',0x82ED)
GL_VERTEX_SHADER_INVOCATIONS=_C('GL_VERTEX_SHADER_INVOCATIONS',0x82F0)
GL_VERTICES_SUBMITTED=_C('GL_VERTICES_SUBMITTED',0x82EE)
@_f
@_p.types(None,_cs.GLenum,ctypes.c_void_p,_cs.GLintptr,_cs.GLsizei,_cs.GLsizei)
def glMultiDrawArraysIndirectCount(mode,indirect,drawcount,maxdrawcount,stride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,ctypes.c_void_p,_cs.GLintptr,_cs.GLsizei,_cs.GLsizei)
def glMultiDrawElementsIndirectCount(mode,type,indirect,drawcount,maxdrawcount,stride):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glPolygonOffsetClamp(factor,units,clamp):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLcharArray,_cs.GLuint,arrays.GLuintArray,arrays.GLuintArray)
def glSpecializeShader(shader,pEntryPoint,numSpecializationConstants,pConstantIndex,pConstantValue):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_gl_spirv'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_gl_spirv',error_checker=_errors._error_checker)
GL_SHADER_BINARY_FORMAT_SPIR_V_ARB=_C('GL_SHADER_BINARY_FORMAT_SPIR_V_ARB',0x9551)
GL_SPIR_V_BINARY_ARB=_C('GL_SPIR_V_BINARY_ARB',0x9552)
@_f
@_p.types(None,_cs.GLuint,arrays.GLcharArray,_cs.GLuint,arrays.GLuintArray,arrays.GLuintArray)
def glSpecializeShaderARB(shader,pEntryPoint,numSpecializationConstants,pConstantIndex,pConstantValue):pass



'''OpenGL extension ATI.texture_mirror_once

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.texture_mirror_once to provide a more 
Python-friendly API

Overview (from the spec)
	
	ATI_texture_mirror_once extends the set of texture wrap modes to 
	include two modes (GL_MIRROR_CLAMP_ATI, GL_MIRROR_CLAMP_TO_EDGE_ATI) 
	that effectively use a texture map twice as large as the original image 
	in which the additional half of the new image is a mirror image of the 
	original image.
	
	This new mode relaxes the need to generate images whose opposite edges
	match by using the original image to generate a matching "mirror image".
	This mode allows the texture to be mirrored only once in the negative
	s, t, and r directions.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/texture_mirror_once.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ATI.texture_mirror_once import *
from OpenGL.raw.GL.ATI.texture_mirror_once import _EXTENSION_NAME

def glInitTextureMirrorOnceATI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_ES3_2_compatibility'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_ES3_2_compatibility',error_checker=_errors._error_checker)
GL_MULTISAMPLE_LINE_WIDTH_GRANULARITY_ARB=_C('GL_MULTISAMPLE_LINE_WIDTH_GRANULARITY_ARB',0x9382)
GL_MULTISAMPLE_LINE_WIDTH_RANGE_ARB=_C('GL_MULTISAMPLE_LINE_WIDTH_RANGE_ARB',0x9381)
GL_PRIMITIVE_BOUNDING_BOX_ARB=_C('GL_PRIMITIVE_BOUNDING_BOX_ARB',0x92BE)
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glPrimitiveBoundingBoxARB(minX,minY,minZ,minW,maxX,maxY,maxZ,maxW):pass



'''OpenGL extension NV.non_square_matrices

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.non_square_matrices to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds support for non-square matrix variables in GLSL shaders.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/non_square_matrices.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.non_square_matrices import *
from OpenGL.raw.GLES2.NV.non_square_matrices import _EXTENSION_NAME

def glInitNonSquareMatricesNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glUniformMatrix2x3fvNV.value size not checked against count*6
glUniformMatrix2x3fvNV=wrapper.wrapper(glUniformMatrix2x3fvNV).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix3x2fvNV.value size not checked against count*6
glUniformMatrix3x2fvNV=wrapper.wrapper(glUniformMatrix3x2fvNV).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix2x4fvNV.value size not checked against count*8
glUniformMatrix2x4fvNV=wrapper.wrapper(glUniformMatrix2x4fvNV).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix4x2fvNV.value size not checked against count*8
glUniformMatrix4x2fvNV=wrapper.wrapper(glUniformMatrix4x2fvNV).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix3x4fvNV.value size not checked against count*12
glUniformMatrix3x4fvNV=wrapper.wrapper(glUniformMatrix3x4fvNV).setInputArraySize(
    'value', None
)
# INPUT glUniformMatrix4x3fvNV.value size not checked against count*12
glUniformMatrix4x3fvNV=wrapper.wrapper(glUniformMatrix4x3fvNV).setInputArraySize(
    'value', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.blend_func_extended

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.blend_func_extended to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides an ES version of the ARB_blend_func_extended
	functionality.
	
	Traditional OpenGL includes fixed-function blending that combines
	source colors with the existing content of a render buffer in
	a variety of ways.  A number of extensions have enhanced this
	functionality by adding further sources of blending weights and
	methods to combine them. However, the inputs to the fixed-function
	blending units are constrained to a source color (as output from
	fragment shading), destination color (as the current content of the
	frame buffer) or constants that may be used in their place.
	
	This extension adds new blending functions whereby a fragment
	shader may output two colors, one of which is treated as the
	source color, and the other used as a blending factor for either
	source or destination colors.  Furthermore, this extension increases
	orthogonality by allowing the SRC_ALPHA_SATURATE function to be used
	as the destination weight.
	
	Because of the limitations of the OpenGL ES 2.0 shading language,
	new built-in variables (gl_SecondaryFragColorEXT,
	gl_SecondaryFragDataEXT) are added to the ES 1.00 shading language
	rather than introduce more complex features for user-defined fragment
	outputs.  Because such built-in variable are deprecated in ES 3.0,
	these variables are NOT available in the OpenGL ES 3.xx shading
	language verisons.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/blend_func_extended.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.blend_func_extended import *
from OpenGL.raw.GLES2.EXT.blend_func_extended import _EXTENSION_NAME

def glInitBlendFuncExtendedEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glBindFragDataLocationEXT.name size not checked against 'name'
glBindFragDataLocationEXT=wrapper.wrapper(glBindFragDataLocationEXT).setInputArraySize(
    'name', None
)
# INPUT glGetProgramResourceLocationIndexEXT.name size not checked against 'name'
glGetProgramResourceLocationIndexEXT=wrapper.wrapper(glGetProgramResourceLocationIndexEXT).setInputArraySize(
    'name', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_MESA_framebuffer_flip_y'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_MESA_framebuffer_flip_y',error_checker=_errors._error_checker)
GL_FRAMEBUFFER_FLIP_Y_MESA=_C('GL_FRAMEBUFFER_FLIP_Y_MESA',0x8BBB)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glFramebufferParameteriMESA(target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetFramebufferParameterivMESA(target,pname,params):pass



'''OpenGL extension AMD.shader_ballot

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.shader_ballot to provide a more 
Python-friendly API

Overview (from the spec)
	
	The extensions ARB_shader_group_vote and ARB_shader_ballot introduced the
	concept of sub-groups and a set of operations that allow data exchange
	across shader invocations within a sub-group.
	
	This extension further extends the capabilities of these extensions with
	additional sub-group operations.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/shader_ballot.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.shader_ballot import *
from OpenGL.raw.GL.AMD.shader_ballot import _EXTENSION_NAME

def glInitShaderBallotAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_IMG_framebuffer_downsample'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_IMG_framebuffer_downsample',error_checker=_errors._error_checker)
GL_DOWNSAMPLE_SCALES_IMG=_C('GL_DOWNSAMPLE_SCALES_IMG',0x913E)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_SCALE_IMG=_C('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_SCALE_IMG',0x913F)
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_AND_DOWNSAMPLE_IMG=_C('GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_AND_DOWNSAMPLE_IMG',0x913C)
GL_NUM_DOWNSAMPLE_SCALES_IMG=_C('GL_NUM_DOWNSAMPLE_SCALES_IMG',0x913D)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint)
def glFramebufferTexture2DDownsampleIMG(target,attachment,textarget,texture,level,xscale,yscale):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint)
def glFramebufferTextureLayerDownsampleIMG(target,attachment,texture,level,layer,xscale,yscale):pass



'''OpenGL extension OES.extended_matrix_palette

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.extended_matrix_palette to provide a more 
Python-friendly API

Overview (from the spec)
	
	The OES_matrix_palette extension added the ability to support vertex skinning 
	in OpenGL ES.  One issue with OES_matrix_palette is that the minimum size of 
	the matrix palette is very small.  This leads to applications having to break 
	geometry into smaller primitive sets called via. glDrawElements.  This has an 
	impact on the overall performance of the OpenGL ES implementation.  In general, 
	hardware implementations prefer primitive packets with as many triangles as 
	possible.  The default minimum size defined in OES_matrix_palette is not 
	sufficient to allow this.  The OES_extended_matrix_palette extension increases 
	this minimum from 9 to 32.  
	
	    Another issue is that it is very difficult for ISVs to handle different 
	    size matrix palettes as it affects how they store their geometry 
	    in the database - may require multiple representations which is
	    not really feasible.  So the minimum size is going to be what most ISVs
	    will use.
	
	    By extending the minimum size of the matrix palette, we remove this
	    fragmentation and allow applications to render geometry with minimal
	    number of calls to glDrawElements or glDrawArrays.  The OpenGL ES
	    implementation can support this without requiring any additional hardware
	    by breaking the primitive, plus it gives implementations the flexibility
	    to accelerate with a bigger matrix palette if they choose to do so.
	
	    Additionally, feedback has also been received to increase the number of
	    matrices that are blend per vertex from 3 to 4.  The OES_extended_matrix_palette
	    extension increases the minium number of matrices / vertex to 4.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/extended_matrix_palette.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.extended_matrix_palette import *
from OpenGL.raw.GLES1.OES.extended_matrix_palette import _EXTENSION_NAME

def glInitExtendedMatrixPaletteOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_AMD_framebuffer_multisample_advanced'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_AMD_framebuffer_multisample_advanced',error_checker=_errors._error_checker)
GL_MAX_COLOR_FRAMEBUFFER_SAMPLES_AMD=_C('GL_MAX_COLOR_FRAMEBUFFER_SAMPLES_AMD',0x91B3)
GL_MAX_COLOR_FRAMEBUFFER_STORAGE_SAMPLES_AMD=_C('GL_MAX_COLOR_FRAMEBUFFER_STORAGE_SAMPLES_AMD',0x91B4)
GL_MAX_DEPTH_STENCIL_FRAMEBUFFER_SAMPLES_AMD=_C('GL_MAX_DEPTH_STENCIL_FRAMEBUFFER_SAMPLES_AMD',0x91B5)
GL_NUM_SUPPORTED_MULTISAMPLE_MODES_AMD=_C('GL_NUM_SUPPORTED_MULTISAMPLE_MODES_AMD',0x91B6)
GL_RENDERBUFFER_STORAGE_SAMPLES_AMD=_C('GL_RENDERBUFFER_STORAGE_SAMPLES_AMD',0x91B2)
GL_SUPPORTED_MULTISAMPLE_MODES_AMD=_C('GL_SUPPORTED_MULTISAMPLE_MODES_AMD',0x91B7)
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glNamedRenderbufferStorageMultisampleAdvancedAMD(renderbuffer,samples,storageSamples,internalformat,width,height):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glRenderbufferStorageMultisampleAdvancedAMD(target,samples,storageSamples,internalformat,width,height):pass



'''OpenGL extension KHR.context_flush_control

This module customises the behaviour of the 
OpenGL.raw.GL.KHR.context_flush_control to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL and OpenGL ES have long supported multiple contexts. The
	semantics of switching contexts is generally left to window system
	binding APIs such as WGL, GLX and EGL. Most of these APIs (if not all)
	specify that when the current context for a thread is changed, the
	outgoing context performs an implicit flush of any commands that have
	been issued to that point. OpenGL and OpenGL ES define a flush as
	sending any pending commands for execution and that this action will
	result in their completion in finite time.
	
	This behavior has subtle consequences. For example, if an application is
	rendering to the front buffer and switches contexts, it may assume that
	any rendering performed thus far will eventually be visible to the user.
	With recent introduction of shared memory buffers, there become inumerable
	ways in which applications may observe side effects of an implicit flush
	(or lack thereof).
	
	In general, a full flush is not the desired behavior of the application.
	Nonetheless, applications that switch contexts frequently suffer the
	performance consequences of this unless implementations make special
	considerations for them, which is generally untenable.
	
	The EGL, GLX, and WGL extensions add new context creation parameters
	that allow an application to specify the behavior that is desired when a
	context is made non-current, and specifically to opt out of the implicit
	flush behavior. The GL extension allows querying the context flush
	behavior.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/KHR/context_flush_control.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.KHR.context_flush_control import *
from OpenGL.raw.GL.KHR.context_flush_control import _EXTENSION_NAME

def glInitContextFlushControlKHR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension IMG.texture_compression_pvrtc

This module customises the behaviour of the 
OpenGL.raw.GLES1.IMG.texture_compression_pvrtc to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides additional texture compression functionality
	specific to Imagination Technologies PowerVR Texture compression format
	(called PVRTC) subject to all the requirements and limitations described 
	by the OpenGL 1.3 specifications.
	
	This extension supports 4 and 2 bit per pixel texture compression
	formats. Because the compression of PVRTC is very CPU intensive,
	it is not appropriate to carry out compression on the target
	platform. Therefore this extension only supports the loading of
	compressed texture data.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/IMG/texture_compression_pvrtc.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.IMG.texture_compression_pvrtc import *
from OpenGL.raw.GLES1.IMG.texture_compression_pvrtc import _EXTENSION_NAME

def glInitTextureCompressionPvrtcIMG():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.texture_add_env

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.texture_add_env to provide a more 
Python-friendly API

Overview (from the spec)
	
	    New texture environment function ADD is supported with the following 
	    equation: 
		 		Cv = Cf + CcCt + Cb
	
	    New function may be specified by calling glTexEnv with GL_ADD token.
	    New parameter Cb (bias) may be specified by calling TexEnv with
	    TEXTURE_ENV_BIAS_SGIX token. 
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/texture_add_env.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.texture_add_env import *
from OpenGL.raw.GL.SGIX.texture_add_env import _EXTENSION_NAME

def glInitTextureAddEnvSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.texture_shader2

This module customises the behaviour of the 
OpenGL.raw.GL.NV.texture_shader2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension extends the NV_texture_shader functionality to
	support texture shader operations for 3D textures.
	
	See the NV_texture_shader extension for information about the
	texture shader operational model.
	
	The two new texture shader operations are:
	
	<conventional textures>
	
	22.  TEXTURE_3D - Accesses a 3D texture via (s/q,t/q,r/q).
	
	<dot product textures>
	
	23.  DOT_PRODUCT_TEXTURE_3D_NV - When preceded by two DOT_PRODUCT_NV
	     programs in the previous two texture shader stages, computes a
	     third similar dot product and composes the three dot products
	     into (s,t,r) texture coordinate set to access a 3D non-projective
	     texture.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/texture_shader2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.texture_shader2 import *
from OpenGL.raw.GL.NV.texture_shader2 import _EXTENSION_NAME

def glInitTextureShader2NV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.point_size_array

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.point_size_array to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension extends how points and point sprites are rendered
	by allowing an array of point sizes instead of a fixed input point
	size given by PointSize.  This provides flexibility for applications
	to do particle effects.
	
	The vertex arrays will be extended to include a point size array.
	The point size array can be enabled/disabled via POINT_SIZE_ARRAY_OES.
	
	The point size array, if enabled, controls the sizes used to render
	points and point sprites.  If point size array is enabled, the point
	size defined by PointSize is ignored.  The point sizes supplied in the 
	point size arrays will be the sizes used to render both points and
	point sprites.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/point_size_array.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.point_size_array import *
from OpenGL.raw.GLES1.OES.point_size_array import _EXTENSION_NAME

def glInitPointSizeArrayOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glPointSizePointerOES.pointer size not checked against 'type,stride'
glPointSizePointerOES=wrapper.wrapper(glPointSizePointerOES).setInputArraySize(
    'pointer', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.compute_variable_group_size

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.compute_variable_group_size to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows applications to write generic compute shaders that
	operate on workgroups with arbitrary dimensions.  Instead of specifying a
	fixed workgroup size in the compute shader, an application can use a
	compute shader using the /local_size_variable/ layout qualifer to indicate
	a variable workgroup size.  When using such compute shaders, the new
	command DispatchComputeGroupSizeARB should be used to specify both a
	workgroup size and workgroup count.
	
	In this extension, compute shaders with fixed group sizes must be
	dispatched by DispatchCompute and DispatchComputeIndirect.  Compute
	shaders with variable group sizes must be dispatched via
	DispatchComputeGroupSizeARB.  No support is provided in this extension for
	indirect dispatch of compute shaders with a variable group size.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/compute_variable_group_size.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.compute_variable_group_size import *
from OpenGL.raw.GL.ARB.compute_variable_group_size import _EXTENSION_NAME

def glInitComputeVariableGroupSizeARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NVX.progress_fence

This module customises the behaviour of the 
OpenGL.raw.GL.NVX.progress_fence to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension uses the concept of GL semaphores as defined in 
	GL_EXT_semaphore to better coordinate operations between multiple
	GPU command streams. A semaphore type called "progress fence" is 
	derived from the GL semaphore. The progress fence semaphore is
	created by CreateProgressFenceNVX() returning the name of a newly
	created semaphore object. Like other semaphores, these are signaled
	by the GL server. Each signal operation is queued in the GPU command
	stream with an associated fence value that is written to the semaphore
	at the completion of a signal operation. 
	
	A GL server wait can be added to the command stream using WaitSemaphoreui64NVX.
	This blocks the GPU until the progress fence semaphore reaches or exceeds the 
	specified fence value.
	
	A GL client wait can be initiated using ClientWaitSemaphoreui64NVX. 
	This blocks the CPU until the specified fence value is reached.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NVX/progress_fence.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NVX.progress_fence import *
from OpenGL.raw.GL.NVX.progress_fence import _EXTENSION_NAME

def glInitProgressFenceNVX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glSignalSemaphoreui64NVX.fenceValueArray size not checked against fenceObjectCount
# INPUT glSignalSemaphoreui64NVX.semaphoreArray size not checked against fenceObjectCount
glSignalSemaphoreui64NVX=wrapper.wrapper(glSignalSemaphoreui64NVX).setInputArraySize(
    'fenceValueArray', None
).setInputArraySize(
    'semaphoreArray', None
)
# INPUT glWaitSemaphoreui64NVX.fenceValueArray size not checked against fenceObjectCount
# INPUT glWaitSemaphoreui64NVX.semaphoreArray size not checked against fenceObjectCount
glWaitSemaphoreui64NVX=wrapper.wrapper(glWaitSemaphoreui64NVX).setInputArraySize(
    'fenceValueArray', None
).setInputArraySize(
    'semaphoreArray', None
)
# INPUT glClientWaitSemaphoreui64NVX.fenceValueArray size not checked against fenceObjectCount
# INPUT glClientWaitSemaphoreui64NVX.semaphoreArray size not checked against fenceObjectCount
glClientWaitSemaphoreui64NVX=wrapper.wrapper(glClientWaitSemaphoreui64NVX).setInputArraySize(
    'fenceValueArray', None
).setInputArraySize(
    'semaphoreArray', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.subsample

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.subsample to provide a more 
Python-friendly API

Overview (from the spec)
	
		Many video image formats and compression techniques utilize
		various component subsamplings, so it is necessary to provide a
		mechanism to specify the up- and down-sampling of components as
		pixel data is drawn from and read back to the client. Though
		subsampled components are normally associated with the video
		color space, YCrCb, use of subsampling in OpenGL does not imply
		a specific color space.
	
		This extension defines new pixel storage modes that are used in
		the conversion of image data to and from component subsampled
		formats on the client side. The extension defines a new pixel
		storage mode to specify these sampling patterns, there are
		three legal values (PIXEL_SUBSAMPLE_4444_SGIX, 
		PIXEL_SUBSAMPLE_4242_SGIX, and 
	        PIXEL_SUBSAMPLE_2424_SGIX).
	
		When pixel data is received from the client and an unpacking
		upsampling mode other than PIXEL_SUBSAMPLE_4444_SGIX is 
	        specified, the upsampling is performed via replication, 
	        unless otherwise specified by RESAMPLE_SGIX.
	
		Similarly, when pixel data is read back to the client and a
		packing downsampling mode other than 
	        PIXEL_SUBSAMPLE_4444_SGIX is specified, downsampling is 
	        performed via simple component decimation (point sampling), 
	        unless otherwise specified by RESAMPLE_SGIX.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/subsample.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.subsample import *
from OpenGL.raw.GL.SGIX.subsample import _EXTENSION_NAME

def glInitSubsampleSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.sparse_texture

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.sparse_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	Recent advances in application complexity and a desire for higher
	resolutions have pushed texture sizes up considerably. Often, the amount
	of physical memory available to a graphics processor is a limiting factor
	in the performance of texture-heavy applications. Once the available
	physical memory is exhausted, paging may occur bringing performance down
	considerably - or worse, the application may fail. Nevertheless, the amount
	of address space available to the graphics processor has increased to the
	point where many gigabytes - or even terabytes of address space may be
	usable even though that amount of physical memory is not present.
	
	This extension allows the separation of the graphics processor's address
	space (reservation) from the requirement that all textures must be
	physically backed (commitment). This exposes a limited form of
	virtualization for textures. Use cases include sparse (or partially
	resident) textures, texture paging, on-demand and delayed loading of
	texture assets and application controlled level of detail.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/sparse_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.sparse_texture import *
from OpenGL.raw.GLES2.EXT.sparse_texture import _EXTENSION_NAME

def glInitSparseTextureEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.conservative_raster

This module customises the behaviour of the 
OpenGL.raw.GL.NV.conservative_raster to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a "conservative" rasterization mode where any pixel
	that is partially covered, even if no sample location is covered, is 
	treated as fully covered and a corresponding fragment will be shaded.
	
	A new control is also added to modify window coordinate snapping 
	precision.
	
	These controls can be used to implement "binning" to a low-resolution
	render target, for example to determine which tiles of a sparse texture
	need to be populated. An app can construct a framebuffer where there is
	one pixel per tile in the sparse texture, and adjust the number of
	subpixel bits such that snapping occurs to the same effective grid as when
	rendering to the sparse texture. Then triangles should cover (at least)
	the same pixels in the low-res framebuffer as they do tiles in the sparse
	texture.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/conservative_raster.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.conservative_raster import *
from OpenGL.raw.GL.NV.conservative_raster import _EXTENSION_NAME

def glInitConservativeRasterNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.shader_clock

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shader_clock to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension exposes a 64-bit monotonically incrementing shader
	counter which may be used to derive local timing information within
	a single shader invocation.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/shader_clock.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.shader_clock import *
from OpenGL.raw.GL.ARB.shader_clock import _EXTENSION_NAME

def glInitShaderClockARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_NV_conservative_raster'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_NV_conservative_raster',error_checker=_errors._error_checker)
GL_CONSERVATIVE_RASTERIZATION_NV=_C('GL_CONSERVATIVE_RASTERIZATION_NV',0x9346)
GL_MAX_SUBPIXEL_PRECISION_BIAS_BITS_NV=_C('GL_MAX_SUBPIXEL_PRECISION_BIAS_BITS_NV',0x9349)
GL_SUBPIXEL_PRECISION_BIAS_X_BITS_NV=_C('GL_SUBPIXEL_PRECISION_BIAS_X_BITS_NV',0x9347)
GL_SUBPIXEL_PRECISION_BIAS_Y_BITS_NV=_C('GL_SUBPIXEL_PRECISION_BIAS_Y_BITS_NV',0x9348)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint)
def glSubpixelPrecisionBiasNV(xbits,ybits):pass



'''OpenGL extension APPLE.texture_max_level

This module customises the behaviour of the 
OpenGL.raw.GLES1.APPLE.texture_max_level to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows an application to specify the maximum (coarsest) 
	mipmap level that may be selected for the specified texture.  This maximum
	level is also used to determine which mip levels are considered when 
	determining texture completeness.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/texture_max_level.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.APPLE.texture_max_level import *
from OpenGL.raw.GLES1.APPLE.texture_max_level import _EXTENSION_NAME

def glInitTextureMaxLevelAPPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION